import UIKit
import WebKit

class JJWebChatViewController: UIViewController {

    private let userContentController = WKUserContentController()
    
    lazy var webView: WKWebView = {
        let config = WKWebViewConfiguration()
        config.allowsInlineMediaPlayback = true
        config.suppressesIncrementalRendering = true
        config.userContentController = userContentController
        let webView = WKWebView(frame: .zero, configuration: config)
        webView.backgroundColor = .clear
        webView.scrollView.isScrollEnabled = false
        webView.isOpaque = false
        webView.uiDelegate = self
        webView.navigationDelegate = self
        return webView
    }()
        
    var randomPort: Int = 0
    
    var currentUrl: String = ""
        
    var actionClosure: ((String)->Void)?
    
    var statusClosure: ((Bool)->Void)?
    
    var loadCompletionClosure: (()->Void)?
    
    var aiStatusClosure: ((Bool)->Void)?
    
    private var allowMicrophoneAccess: Bool = false
    private var allowCameraAccess: Bool = false
        
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
        layoutViews()
        loadUrl(currentUrl)
        
        // info.plist 添加
        // Privacy - Microphone Usage Description
        // Privacy - Photo Library Usage Description
        // Privacy - Photo Library Additions Usage Description
        // Privacy - Camera Usage Description
        
        userContentController.add(self, name: "appDownFile")
        userContentController.add(self, name: "checkPermission")
    }
    
    private func setupUI() {
        view.backgroundColor = UIColor.white
        view.addSubview(webView)
        
        if #available(iOS 11.0, *) {
            webView.scrollView.contentInsetAdjustmentBehavior = .never
        } else {
            automaticallyAdjustsScrollViewInsets = false
        }
        
        let button = UIButton()
        button.setImage(UIImage(named: "arrow-left"), for: .normal)
        button.addTarget(self, action: #selector(backAction), for: .touchUpInside)
        navigationItem.leftBarButtonItem = UIBarButtonItem(customView: button)
    }
        
    private func layoutViews() {
        webView.frame = CGRect(x: 0, y: 0, width: kScreenWidth, height: kScreenHeight)
    }
    
    func exit() {
        userContentController.removeScriptMessageHandler(forName: "appDownFile")
        userContentController.removeScriptMessageHandler(forName: "checkPermission")
    }
    
    @objc func backAction() {
        print("点击返回")
        navigationController?.popViewController(animated: true)
    }
    
    deinit {
        print("deinit")
    }
}

extension JJWebChatViewController {
    func loadUrl(_ urlStr: String?) {
        guard let urlStr = urlStr else { return }
        guard let url = URL(string: urlStr) else { return }
        let request = URLRequest(url: url)
        webView.load(request)
    }
}

extension JJWebChatViewController: WKUIDelegate {
    func webView(_ webView: WKWebView, runJavaScriptAlertPanelWithMessage message: String, initiatedByFrame frame: WKFrameInfo, completionHandler: @escaping () -> Void) {
        
    }
    
    func webView(_ webView: WKWebView, runJavaScriptConfirmPanelWithMessage message: String, initiatedByFrame frame: WKFrameInfo, completionHandler: @escaping (Bool) -> Void) {
        
    }
}

extension JJWebChatViewController {
    func refreshWebView() {
        webView.reload()
    }
}

extension JJWebChatViewController: WKNavigationDelegate {
    func webView(_ webView: WKWebView, didStartProvisionalNavigation navigation: WKNavigation!) {
        print("webView ==== 开始加载")
    }
    
    func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        print("webView ==== 加载完成")
    }
    
    func webView(_ webView: WKWebView, didFailProvisionalNavigation navigation: WKNavigation!, withError error: Error) {
        print("webView ==== 加载错误 \(error)")
    }
    
    @available(iOS 15.0, *)
    func webView(_ webView: WKWebView, requestMediaCapturePermissionFor origin: WKSecurityOrigin, initiatedByFrame frame: WKFrameInfo, type: WKMediaCaptureType, decisionHandler: @escaping (WKPermissionDecision) -> Void) {
        switch type {
        case .camera:
            decisionHandler(allowCameraAccess ? .grant : .deny)
        case .microphone:
            decisionHandler(allowMicrophoneAccess ? .grant : .deny)
        case .cameraAndMicrophone:
            decisionHandler(allowCameraAccess && allowMicrophoneAccess ? .grant : .deny)
        @unknown default:
            decisionHandler(.deny)
        }
    }
}

extension JJWebChatViewController: WKScriptMessageHandler {
    func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        
        print("webView ==== 收到消息 \(message.name) == \(message.body)")

        switch message.name {
        case "appDownFile":
            guard let dict = message.body as? [String: Any] else { return }
            
            guard let functionStr = dict["callback"] as? String else { return }
            
            guard let params = dict["params"] as? [String: Any] else { return }
                    
            guard let fileUrl = params["fileUrl"] as? String else { return }
            
            downloadFile(from: fileUrl, to: functionStr)
        case "checkPermission":
            
            guard let dict = message.body as? [String: Any] else { return }
            
            guard let functionStr = dict["callback"] as? String else { return }
            
            guard let params = dict["params"] as? [String: Any] else { return }
                    
            guard let permission = params["checkPermission"] as? String else { return }

            checkPermission(to: functionStr, type: permission)
        default: break
        }
    }
    
    private func downloadFile(from urlString: String, to functionStr: String) {
        guard let url = URL(string: urlString) else {
            sendDownloadCallback(to: functionStr, success: false, error: "Invalid URL")
            return
        }
        
        let task = URLSession.shared.downloadTask(with: url) { localURL, response, error in
            if let error = error {
                self.sendDownloadCallback(to: functionStr, success: false, error: error.localizedDescription)
                return
            }
            
            guard let localURL = localURL else {
                self.sendDownloadCallback(to: functionStr, success: false, error: "Download failed")
                return
            }
            
            // 获取文件管理器
            let fileManager = FileManager.default
            
            // 获取应用程序的Documents目录
            guard let documentsPath = fileManager.urls(for: .documentDirectory, in: .userDomainMask).first else {
                self.sendDownloadCallback(to: functionStr, success: false, error: "Cannot access documents directory")
                return
            }
            
            // 生成目标文件路径
            let fileName = url.lastPathComponent
            let destinationURL = documentsPath.appendingPathComponent(fileName)
            
            do {
                // 如果目标文件已存在，先删除
                if fileManager.fileExists(atPath: destinationURL.path) {
                    try fileManager.removeItem(at: destinationURL)
                }
                
                // 将下载的文件移动到目标位置
                try fileManager.moveItem(at: localURL, to: destinationURL)
                self.sendDownloadCallback(to: functionStr, success: true, filePath: destinationURL.path)
            } catch {
                self.sendDownloadCallback(to: functionStr, success: false, error: error.localizedDescription)
            }
        }
        task.resume()
    }
    
    private func sendDownloadCallback(to functionStr: String, success: Bool, filePath: String? = nil, error: String? = nil) {
        DispatchQueue.main.async {
            let script = "\(functionStr)(\(success))"
            print("发送JS消息 === \(script)")
            self.webView.evaluateJavaScript(script, completionHandler: nil)
        }
    }
    
    private func checkPermission(to functionStr: String, type: String) {
        switch type {
        case "record_audio":
            JJPermission.checkMicrophonePermission { [weak self](isOpen) in
                self?.allowMicrophoneAccess = isOpen
                self?.sendNormalCallback(to: functionStr, success: isOpen)
            }
            break
        case "file":
            JJPermission.checkPhotoLibraryPermission { [weak self](isOpen) in
                self?.sendNormalCallback(to: functionStr, success: isOpen)
            }
            break
        case "camera":
            JJPermission.checkCameraPermission { [weak self](isOpen) in
                self?.allowCameraAccess = isOpen
                self?.sendNormalCallback(to: functionStr, success: isOpen)
            }
            break
        default:
            break
        }
    }
    
    private func sendNormalCallback(to functionStr: String, success: Bool) {
        DispatchQueue.main.async {
            let script = "\(functionStr)(\(success))"
            print("发送JS消息 === \(script)")
            self.webView.evaluateJavaScript(script, completionHandler: nil)
        }
    }
}
